

/**
* This file is part of license combination gpl version 3 license and eCos.
* The corresponding license terms are below.
*
* gpl version 3 Licence:
*
* The file were developed during the student thesis "Data Aggregation 
* using TinyIPFIX in Wireless Sensor Networks" of Benjamin Ertl and is 
* included in the dissertation "Secure Data Transmission in Wireless 
* Sensor Networks" by Corinna Schmitt during employment at the Technische
* Universitt Mnchen, Department Computer Science, Chair Network 
* Architectures and Services (Germany).
* Copyright (C) 2013  
* Authors: Benjamin Ertl and Corinna Schmitt (schmitt[at]net.in.tum.de)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, version 3 of the License
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
*
*
* eCos Licence: (http://ecos.sourceware.org)
*
* This file is part of eCos, the Embedded Configurable Operating System. 
* Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 
* 2007, 2008, 2009 Free Software Foundation, Inc. 
*
* eCos is free software; you can redistribute it and/or modify it under 
* the terms of the GNU General Public License as published by the Free 
* Software Foundation; either version 3 any later version. 
* 
* eCos is distributed in the hope that it will be useful, but WITHOUT ANY 
* WARRANTY; without even the implied warranty of MERCHANTABILITY or 
* FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
* more details. 
* 
* You should have received a copy of the GNU General Public License along 
* with eCos; if not, write to the Free Software Foundation, Inc., 51 
* Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. 
*
* As a special exception, if other files instantiate templates or use 
* macros or inline functions from this file, or you compile this file and * link it with other works to produce a work based on this file, this file 
* does not by itself cause the resulting work to be covered by the GNU 
* General Public License. However the source code for this file must still 
* be made available in accordance with section (3) of the GNU General 
* Public License v2. 
* 
* This exception does not invalidate any other reasons why a work based on 
* this file might be covered by the GNU General Public License.
*
*/


package xml;

import ipfix.Field;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

public class TemplateEnricher extends DefaultHandler{
	String qualifier = "*|*|*";
	String unignore = null;
	boolean nosend = false;
	String fieldName = null, dataType = null, fieldID = null, enterpriseNumber = null;
	String characterTarget = null;
	
	
	@Override
	public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
		if (unignore != null) return;
		
		if(qName.equals("match")){
			qualifier = "";
			String source = attributes.getValue("source"), observation = attributes.getValue("domain"), templateID = attributes.getValue("templateid");
			
			if(templateID != null) qualifier += templateID; else qualifier += "*";			
			qualifier+="|";
			if(source != null) qualifier += source; else qualifier += "*";
			qualifier+="|";
			if(observation != null) qualifier += observation; else qualifier += "*";
			
		} else if(qName.equals("field")) {
			// set all field variables back to default values:
			this.nosend = false;
			fieldName = null;
			dataType = null;
			fieldID = null;
			enterpriseNumber = null;
			
			String nosend = attributes.getValue("nosend");
			if(nosend != null){
				this.nosend = Boolean.parseBoolean(nosend);
			}
		} else if(qName.equals("name")){
			fieldName = new String();
			characterTarget = fieldName;			
		} else if(qName.equals("fieldID")){
			fieldID = new String();
			characterTarget = fieldID;
		} else if(qName.equals("enterpriseNumber")){
			enterpriseNumber = new String();
			characterTarget = enterpriseNumber;
		} else if(qName.equals("dataType")){
			dataType = new String();
			characterTarget = dataType;
		} else if(qName.equals("configuration")){
			
		} else {
			System.err.printf("Ignored XML Element named %s.\n", qName);
			//remember the name so that the whole subtree will be ignored
			unignore = qName;
		}
	}
	
	@Override
	public void characters(char[] ch, int start, int length) throws SAXException {
		if(characterTarget == null) return;
		
		char[] buff = new char[length];
		String strbuff;
		System.arraycopy(ch, start, buff, 0, length);
		strbuff = new String(buff);
		strbuff = strbuff.trim();
		if(characterTarget == fieldID) fieldID = strbuff;
		else if(characterTarget == enterpriseNumber) enterpriseNumber = strbuff;
		else if(characterTarget == fieldName) fieldName = strbuff;
		else if(characterTarget == dataType) dataType = strbuff;
		return;
	}
	
	@Override
	public void endElement(String uri, String localName, String qName) throws SAXException {
		if(unignore != null){
			// we have found the end of a ignored subtree
			if (qName.equals(unignore)) unignore = null;
			return;
		}
		if(qName.equals("field")){
			Field prototype = new Field(0, 0, 0);
			int fieldID, enterpriseNumber;
			try {
				fieldID = Integer.parseInt(this.fieldID);
				prototype.setID(fieldID);
			} catch (Exception e) {
				return;
			}
			try {
				enterpriseNumber = Integer.parseInt(this.enterpriseNumber);
				prototype.setEnterpriseID(enterpriseNumber);
			} catch (Exception e) {
				return;
			}
			prototype.setSendout(!nosend);
			if(fieldName != null){
				prototype.setName(fieldName);
			}
			if(dataType != null){
				if(dataType.equals("int8") || dataType.equals("int16") || dataType.equals("int32") || dataType.equals("int64") || dataType.equals("float") || dataType.equals("blob")){
					prototype.setDatatype(dataType);
				}else {
					System.err.printf("Invalid dataType %s, will treat object as blob\n",dataType);
					prototype.setDatatype("blob");
				}
			}
			try {
				FieldMetadataHeap.getInstance().putFields(qualifier, prototype);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
}