

/**
* This file is part of license combination gpl version 3 license and eCos.
* The corresponding license terms are below.
*
* gpl version 3 Licence:
*
* The file were developed during the student thesis "Data Aggregation 
* using TinyIPFIX in Wireless Sensor Networks" of Benjamin Ertl and is 
* included in the dissertation "Secure Data Transmission in Wireless 
* Sensor Networks" by Corinna Schmitt during employment at the Technische
* Universitt Mnchen, Department Computer Science, Chair Network 
* Architectures and Services (Germany).
* Copyright (C) 2013  
* Authors: Benjamin Ertl and Corinna Schmitt (schmitt[at]net.in.tum.de)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, version 3 of the License
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
*
*
* eCos Licence: (http://ecos.sourceware.org)
*
* This file is part of eCos, the Embedded Configurable Operating System. 
* Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 
* 2007, 2008, 2009 Free Software Foundation, Inc. 
*
* eCos is free software; you can redistribute it and/or modify it under 
* the terms of the GNU General Public License as published by the Free 
* Software Foundation; either version 3 any later version. 
* 
* eCos is distributed in the hope that it will be useful, but WITHOUT ANY 
* WARRANTY; without even the implied warranty of MERCHANTABILITY or 
* FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
* more details. 
* 
* You should have received a copy of the GNU General Public License along 
* with eCos; if not, write to the Free Software Foundation, Inc., 51 
* Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. 
*
* As a special exception, if other files instantiate templates or use 
* macros or inline functions from this file, or you compile this file and * link it with other works to produce a work based on this file, this file 
* does not by itself cause the resulting work to be covered by the GNU 
* General Public License. However the source code for this file must still 
* be made available in accordance with section (3) of the GNU General 
* Public License v2. 
* 
* This exception does not invalidate any other reasons why a work based on 
* this file might be covered by the GNU General Public License.
*
*/


/*
  Title: WSN - Secure comunications with AES algoritms
  Company: University of Trento - Faculty of Computer Science
  @author Nicola Manica (128851 - nicola.manica@gmail.com), 
          Matteo Saloni (130196 - matteo.saloni@gmail.com), 
          Toldo Paolo (128723 - paolo.toldo@gmail.com - www.paolotoldo.it)
*/

#ifndef _AES_H
#define _AES_H

#define AES_ENCRYPT     1
#define AES_DECRYPT     0

/**
 * \brief          AES context structure
 */
typedef struct aes_context
{
    int nr;                     /*!<  number of rounds  */
    uint32_t *rk;          /*!<  AES round keys    */
    uint32_t buf[68];      /*!<  unaligned data    */
}
aes_context;

/**
 * \brief          AES key schedule (encryption)
 *
 * \param ctx      AES context to be initialized
 * \param key      encryption key
 * \param keysize  must be 128, 192 or 256
 */
void aes_setkey_enc( aes_context *ctx, unsigned char *key, int keysize );

/**
 * \brief          AES key schedule (decryption)
 *
 * \param ctx      AES context to be initialized
 * \param key      decryption key
 * \param keysize  must be 128, 192 or 256
 */
void aes_setkey_dec( aes_context *ctx, unsigned char *key, int keysize );

/**
 * \brief          AES-ECB block encryption/decryption
 *
 * \param ctx      AES context
 * \param mode     AES_ENCRYPT or AES_DECRYPT
 * \param input    16-byte input block
 * \param output   16-byte output block
 */
void aes_crypt_ecb( aes_context *ctx,
                    int mode,
                    unsigned char input[16],
                    unsigned char output[16] );

/**
 * \brief          AES-CBC buffer encryption/decryption
 *
 * \param ctx      AES context
 * \param mode     AES_ENCRYPT or AES_DECRYPT
 * \param length   length of the input data
 * \param iv       initialization vector (updated after use)
 * \param input    buffer holding the input data
 * \param output   buffer holding the output data
 */
void aes_crypt_cbc( aes_context *ctx,
                    int mode,
                    int length,
                    unsigned char iv[16],
                    unsigned char *input,
                    unsigned char *output );

/**
 * \brief          AES-CFB buffer encryption/decryption
 *
 * \param ctx      AES context
 * \param mode     AES_ENCRYPT or AES_DECRYPT
 * \param length   length of the input data
 * \param iv_off   offset in IV (updated after use)
 * \param iv       initialization vector (updated after use)
 * \param input    buffer holding the input data
 * \param output   buffer holding the output data
 */
void aes_crypt_cfb( aes_context *ctx,
                    int mode,
                    int length,
                    int *iv_off,
                    unsigned char iv[16],
                    unsigned char *input,
                    unsigned char *output );

/**
 * \brief          Checkup routine
 *
 * \return         0 if successful, or 1 if the test failed
 */
int aes_self_test( int verbose );

#endif /* aes.h */
